/*
 * polar.c
 * $Id: polar.c,v 1.1.1.1 2001/08/15 06:23:14 cegger Exp $
 *
 * by Steve Cheng (2000-10-15)
 *
 * Simple LibGGI demo that graphs polar(*) functions.
 * For simplicity, this program does not include an expression parser, so
 * the code must be changed and recompiled to generate new graphs.
 *
 * Verbosely commented but not strictly layered in design 
 * to aid in understanding LibGGI programming.
 * 
 * (*) The independent variable is the angle and the dependent variable
 *     is the radius, both composing polar coordinates.  Some math whiz
 *     correct my terminology here...
 *
 *   This software is placed in the public domain and can be used freely
 *   for any purpose. It comes without any kind of warranty, either
 *   expressed or implied, including, but not limited to the implied
 *   warranties of merchantability or fitness for a particular purpose.
 *   Use it at your own risk. the author is not responsible for any damage
 *   or consequences raised by use or inability to use this program.
 *   
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <ggi/ggi.h>

/* This seems to be an extension to ANSI C, so define it
 * if not already in math.h */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

/* Some parameters */
double scale_x = 2.0;		/* pixels per unit */
double scale_y = 2.0;

double angle_start = 0;		/* lower bound for independent variable */
double angle_stop = 16*2*M_PI;	/* upper bound for independent variable */
double angle_step = +0.001;	/* independent variable increments */


/* LibGGI visual info */
ggi_visual_t vis;
ggi_mode vis_mode;

/* Different colors */
ggi_pixel background_pixelvalue;
ggi_pixel axis_pixelvalue;
ggi_pixel plot_pixelvalue;

/* Axis location */
int center_x;
int center_y;


/*
 * Start up LibGGI.
 */
void init_visual(void)
{
	ggi_color col;
	
	/* GGI initialization mantra */
	if(ggiInit() < 0) {
		fprintf(stderr, "ggiInit() failed!\n");
		exit(1);
	}

	/* Open default target */
	vis = ggiOpen(NULL);
	if(vis == NULL) {
		ggiPanic("Cannot open default visual!\n");
	}

	/* Set the target's default mode or whatever specified
	 * by user in GGI_DEFMODE */
	if(ggiSetGraphMode(vis,
		GGI_AUTO, GGI_AUTO, GGI_AUTO, GGI_AUTO, GT_AUTO) < 0)
	{
		ggiPanic("Cannot set default mode!\n");
	}

	/* Read back what sort of mode we got */
	ggiGetMode(vis, &vis_mode);

	/* Axis at center */
	center_x = vis_mode.virt.x / 2;
	center_y = vis_mode.virt.y / 2;
		
	/* Ensure that we have some basic colors when we
	 * are on a palettized mode */
	ggiSetColorfulPalette(vis);

	/* A few colors used for the graph.
	 */
	col.r = 0xFFFF;	/* white */
	col.g = 0xFFFF;
	col.b = 0xFFFF;
	plot_pixelvalue = ggiMapColor(vis, &col);

	col.r = 0x0;	/* black */
	col.g = 0x0;
	col.b = 0x0;
	background_pixelvalue = ggiMapColor(vis, &col);

	col.r = 0xB000;	/* red */
	col.g = 0x0;
	col.b = 0x0;
	axis_pixelvalue = ggiMapColor(vis, &col);
}

/*
 * Clears the screen and draws the axes.
 */
void setup_graph(void)
{
	ggiSetGCForeground(vis, background_pixelvalue);
	ggiFillscreen(vis);

	ggiSetGCForeground(vis, axis_pixelvalue);
	ggiDrawHLine(vis, 0, center_y, vis_mode.virt.x);
	ggiDrawVLine(vis, center_x, 0, vis_mode.virt.y);
}

void graph(void)
{
	double angle;
	double radius;
	int x, y;

	ggiSetGCForeground(vis, plot_pixelvalue);

	for(angle = angle_start; angle < angle_stop; angle += angle_step)
	{
		radius = cos(16 * angle);
		
		/* Other things you might want to try */
		/* radius = angle/2/M_PI; */
		/* radius = angle/2/M_PI - 4; */
		/* radius = pow(angle, 2); */
		/* radius = sqrt(angle); */
		/* radius = pow(angle, -1); */
		/* radius = -pow(angle/2/M_PI,3) -8*pow(angle/2/M_PI,2) +0.5*angle/2/M_PI +40; */
		/* radius = sqrt(8 * cos(2 * angle)); */
		/* radius = exp(angle/2/M_PI); */
		/* radius = 3 + 3*sin(angle); */
		/* radius = 3 + 3*cos(angle); */
		/* radius = 3 - 3*tan(angle); */
		/* radius = cosh(angle); */
		/* radius = pow(angle/2/M_PI,angle/2/M_PI); */
		/* radius = log(angle); */
		/* radius = erf(angle); */
		/* radius = lgamma(angle); */
		/* radius = tgamma(angle); */
		/* radius = angle*asin(angle/2/M_PI - floor(angle/2/M_PI)); */
		/* radius = angle*(angle/2/M_PI - floor(angle/2/M_PI)); */

		/* Or even: */
		/* radius = (double)random()/(double)RAND_MAX*100.0; */
		/* radius = random()%100; */
		/* radius = 10*angle*(double)random()/(double)RAND_MAX; */

		/* Plot [angle,radius] */
		x = ((radius*cos(angle)) * scale_x) + center_x;
		y = ((radius*sin(angle)) * -scale_y) + center_y;
		ggiDrawPixel(vis, x, y);
	}
}
	
/*
 * Cleanup LibGGI.
 */
void deinit(void)
{
	ggiClose(vis);
	ggiExit();
}

int main(int argc, char *argv[])
{
	init_visual();

	/* Event loop. Possible improvements: dynamically adjusting of
	 * scale, angle bounds; panning.
	 */
	do {
		setup_graph();
		graph();

		ggiGetc(vis);
	} while(0);

	deinit();
	return 0;
}
